//
//  CloudPersistenceManager.swift
//  VoiceMemo
//
//  Created by Screencast on 9/3/16.
//  Copyright © 2016 Treehouse Island, Inc. All rights reserved.
//

import Foundation
import CloudKit

enum PersistenceError: MemoErrorType {
    case saveFailed(description: String)
    case insufficientInformation
    case invalidData
    case queryFailed(description: String, userInfo: [AnyHashable: Any])
    
    var description: String {
        switch self {
        case .saveFailed(let description): return "Save Failed: \(description)"
        case .queryFailed(let description, let userInfo):
            return "Query Failed. Description: \(description), userInfo: \(userInfo.description)"
        default: return String(describing: self)
        }
    }
}

class CloudPersistenceManager {
//    let privateDatabase = CKContainer.defaultContainer().privateCloudDatabase
    let privateDatabase = CKContainer(identifier: "iCloud.com.teamtreehouse.VoiceMemos").privateCloudDatabase
    
    func save(_ memo: Memo, completion: @escaping (Result<Memo>) -> Void) {
        let record = memo.persistableRecord
        
        privateDatabase.save(record, completionHandler: { record, error in
            
            guard let record = record else {
                if let error = error {
                    let persistenceError = PersistenceError.saveFailed(description: error.localizedDescription)
                    completion(.failure(persistenceError))
                } else {
                    let persistenceError = PersistenceError.insufficientInformation
                    completion(.failure(persistenceError))
                }
                
                return
            }
            
            guard let memo = Memo(record: record) else {
                let error = PersistenceError.invalidData
                completion(.failure(error))
                return
            }
            
            completion(.success(memo))
        }) 
    }
    
    func performQuery(_ query: CKQuery, completion: @escaping (Result<[Memo]>) -> Void) {
        privateDatabase.perform(query, inZoneWith: nil) { records, error in
            
            guard let records = records else {
                if let error = error {
                    let persistenceError = PersistenceError.queryFailed(description: error.localizedDescription, userInfo: error._userInfo as! [AnyHashable : Any])
                    completion(.failure(persistenceError))
                } else {
                    let persistenceError = PersistenceError.insufficientInformation
                    completion(.failure(persistenceError))
                }
                return
            }
            
            let memos = records.flatMap { Memo(record: $0) }
            completion(.success(memos))
        }
    }
    
    func fetch(_ recordID: CKRecordID, completion: @escaping (Result<Memo>) -> Void) {
        privateDatabase.fetch(withRecordID: recordID) { record, error in
            guard let record = record else {
                if let error = error {
                    let persistenceError = PersistenceError.queryFailed(description: error.localizedDescription, userInfo: error._userInfo as! [AnyHashable : Any])
                    completion(.failure(persistenceError))
                } else {
                    let persistenceError = PersistenceError.insufficientInformation
                    completion(.failure(persistenceError))
                }
                
                return
            }
            
            guard let memo = Memo(record: record) else {
                let error = PersistenceError.invalidData
                completion(.failure(error))
                return
            }
            
            completion(.success(memo))
        }
    }
    
}
































